drop TYPE IF EXISTS getProductStock_type CASCADE;
CREATE TYPE getProductStock_type AS (
	s_real numeric,
	s_futuro numeric,
	s_disponible numeric,
	s_asignado numeric,
	s_pendiente numeric
);
CREATE OR REPLACE FUNCTION getProductStock(product_id integer, stock integer) 
RETURNS SETOF getProductStock_type AS $$
DECLARE 
	r1 getProductStock_type%ROWTYPE;
	r2 record;
	r3 record;
	r4 record;
	r5 record;
	r record;
	query varchar;
	s_real numeric := 0.0;
	s_futuro numeric := 0.0;
	s_virtual numeric := 0.0;
	s_disponible numeric := 0.0;
	s_asignado numeric := 0.0;
	s_pendiente numeric := 0.0;
BEGIN
	-- Busca producto en stock real
	FOR r2 IN EXECUTE '
		SELECT 
			x.product_uom, 
			SUM(x.coeff * x.product_qty) as product_qty, 
			x.loc_id as location_id
		FROM (
			SELECT 1.0 as coeff, product_uom, location_dest_id as loc_id,
				sum(product_qty) AS product_qty
			FROM stock_move
			WHERE location_dest_id in (12) AND
				location_id != location_dest_id AND
				product_id = '||product_id||' AND
				state in (''done'')
			GROUP BY location_dest_id, product_uom
			UNION
				SELECT -1.0 as coeff, product_uom, location_id as loc_id,
					sum(product_qty) AS product_qty
				FROM stock_move
				WHERE location_id in (12) AND
					location_id != location_dest_id AND
					product_id = '||product_id||' AND
					state in (''done'')
				GROUP BY location_id, product_uom
		) AS x
		GROUP BY x.loc_id, x.product_uom'
	LOOP
		s_real := r2.product_qty;
	END LOOP;
	-- Busca producto en stock virtual
		FOR r4 IN EXECUTE '
		SELECT 
			x.product_uom, 
			SUM(x.coeff * x.product_qty) as product_qty, 
			x.loc_id as location_id
		FROM (
			SELECT 1.0 as coeff, product_uom, location_dest_id as loc_id,
				sum(product_qty) AS product_qty
			FROM stock_move
			WHERE location_dest_id in (12) AND
				location_id != location_dest_id AND
				product_id = '||product_id||' AND
				state in (''confirmed'', ''waiting'', ''assigned'', ''done'')
			GROUP BY location_dest_id, product_uom
			UNION
				SELECT -1.0 as coeff, product_uom, location_id as loc_id,
					sum(product_qty) AS product_qty
				FROM stock_move
				WHERE location_id in (12) AND
					location_id != location_dest_id AND
					product_id = '||product_id||' AND
					state in (''confirmed'', ''waiting'', ''assigned'', ''done'')
				GROUP BY location_id, product_uom
		) AS x
		GROUP BY x.loc_id, x.product_uom'
	LOOP
		s_virtual := r4.product_qty;
	END LOOP;
	FOR r in execute 'SELECT COALESCE(sum(product_qty), 0) AS product_qty
		FROM stock_move 
		WHERE 
		state=''assigned'' AND 
		location_id = location_dest_id AND 
		location_id =12 AND product_id = '||product_id||';'
	LOOP
		s_disponible := r.product_qty;
	END LOOP;
	FOR r in execute 'SELECT COALESCE(sum(product_qty), 0) AS product_qty
		FROM stock_move 
		WHERE id = '||stock||' AND
		state=''assigned'' AND 
		location_id = location_dest_id AND 
		location_id =12 AND product_id = '||product_id||';'
	LOOP
		s_asignado := r.product_qty;
	END LOOP;

	FOR r IN EXECUTE 'SELECT
			COALESCE(
			CASE
				WHEN t.supply_method = ''produce'' 
				THEN (SELECT m.product_qty
					FROM stock_move m 
					WHERE 
						m.production_id = (SELECT p.id FROM mrp_production p WHERE p.product_id = '||product_id||' AND p.move_prod_id = '||stock||')
						AND m.product_id = '||product_id||' AND m.move_dest_id = '||stock||' AND (state = ''assigned'' or state = ''waiting''))
			ELSE
				(SELECT COALESCE(l.product_qty, 0) AS product_qty 
				FROM purchase_order_line l 
				WHERE l.product_id = '||product_id||' AND l.move_dest_id = '||stock||' AND state in (''draft''))
			END, 0) AS pendiente
		FROM product_product p
		LEFT JOIN product_template t on (p.product_tmpl_id = t.id)
		WHERE p.id = '||product_id||';'
	LOOP
		s_pendiente := r.pendiente;
	END LOOP;
	s_disponible := s_real - s_disponible;
    RETURN QUERY SELECT s_real::numeric, s_virtual::numeric, s_disponible::numeric, s_asignado::numeric, s_pendiente::numeric;
END
$$ language 'plpgsql';
--SELECT * FROM getProductStock(3254, 8466);
