# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
import decimal_precision as dp
import time, datetime

DEFAULTENCODING = 'utf8'
DOCUMENTCOLUMN = 1
PROJECTCOLUMN = 2
REVCOLUMN = 3

REV_FIELDLENGTH = 3

PMPREFIX = 'PM-'
PREFIXLEN = 3

DOCREVPREFIXBOM = ['EQ-', 'MD-', 'MDS', 'PM-', 'PL-', 'PR-', 'PV-', 'MA-', 'DEL', 'DNU', 'DNS']
DOCREVPREFIXCODE = ['MDS', 'PM-', 'PL-', 'PR-', 'PV-', 'MA-', 'DEL', 'DNU', 'DNS']

COMPLEXKEYLIST = ['uom_po_id', 'uom_id', 'categ_id']

MODULODICT = {}

BOMDICT = {}

def uniquify(mylist):
    set = {}
    map(set.__setitem__, mylist, [])
    return set.keys()

def process_zero(exp_field):
    field_value  = str(int(exp_field))
    field_length = REV_FIELDLENGTH
    length       = len(field_value) 
    field_value = '0' * (field_length - length) + field_value
    return field_value

def getDefaultCodeFromRow(row):
    mydoc_code = row[1]
    myprod_code = row[2]
    if str(mydoc_code)[:3] in DOCREVPREFIXCODE:
        return '-'.join([str(myprod_code), process_zero(row[3])])
    else:
        return str(myprod_code)

    

def xls2list(fname, worksheet=1,encoding=DEFAULTENCODING):
    from pyExcelerator import parse_xls
    data = parse_xls(fname, encoding) 
    sheet_name=data[worksheet-1][0].encode(encoding) 
    values = data[worksheet-1][1]
    vdict = {} 
    row_idx_max = 0 
    col_idx_max = 0 
    for row_idx, col_idx in sorted(values.keys()): 
        row_idx_max = max(row_idx,row_idx_max) 
        col_idx_max = max(col_idx,col_idx_max) 
        v = values[(row_idx, col_idx)] 
        if isinstance(v, unicode): 
            v = v.encode(encoding, 'backslashreplace') 
        vdict[(row_idx,col_idx)] = v
    vlist = [] 
    for row in range(row_idx_max+1): 
        vlist.append([]) 
        for col in range(col_idx_max+1): 
            if (row,col) not in vdict: 
                vdict[(row,col)]=None 
            vlist[row].append(vdict[(row,col)]) 
    return vlist 

def getAllWorksheets(fname, encoding=DEFAULTENCODING):
    from pyExcelerator import parse_xls
    WorkSheetList = []
    try:
        for i in range(len(parse_xls(fname, encoding))):
            WorkSheetList.append(xls2list(fname, i+1, encoding))
    except Exception, msg:
        raise osv.except_osv(('Error !'), ('Formato no reconocido: %s' %(msg, )))
    return WorkSheetList

def getCols(mylist, colnumlist):
    res = []
    for row in mylist:
        myrow = []
        for colnum in colnumlist:
            myrow.append(row[colnum])
        res.append(myrow)
    return res

def cleanWorkSheetList(wslist):
    outlist = []
    for ws in wslist:
        try:
            while ws[0].count(None) == len(ws[0]):
                del ws[0]
        except IndexError:
            raise osv.except_osv(('Error !'), ('Existen hojas vacías en el archivo de definición.'))
        startcol = 0
        mycol = getCols(ws, [startcol])
        while mycol.count([None]) == len(mycol):
            startcol += 1
            mycol = getCols(ws, [startcol])
        outlist.append([x[startcol:] for x in ws])
    return outlist


class product_product(osv.osv):
    _inherit = "product.product"
    _columns = {'equitek_docnum': fields.char('Document Number', size=128),
                'name_template': fields.related('product_tmpl_id', 'name', string="Name", type='char', size=254, store=True),
                
                }
product_product()

class product_template(osv.osv):
    _inherit = "product.template"
    _columns = {
                'name': fields.char('Name', size=254, required=True, translate=True, select=True),
        }
    
product_template()

class category_relation(osv.osv):
    #_inherit = 'product.product'
    _name = 'category.relation'
    _auto = True
    #_inherits = {'product.product':'product_id'}

    def _get_uom_id(self, cr, uid, *args):
        cr.execute('select id from product_uom order by id limit 1')
        res = cr.fetchone()
        return res and res[0] or False

    def _default_category(self, cr, uid, context=None):
        if context is None:
            context = {}
        if 'categ_id' in context and context['categ_id']:
            return context['categ_id']
        md = self.pool.get('ir.model.data')
        res = md.get_object_reference(cr, uid, 'product', 'cat0') or False
        return res and res[1] or False

    _defaults = {
        'type': lambda *a: 'product',
        'list_price': lambda *a: 1,
        'cost_method': lambda *a: 'standard',
        'list_price': lambda *a: 1,
        'sale_ok': lambda *a: 1,
        'purchase_ok': lambda *a: 1,
        'procure_method': lambda *a: 'make_to_order',
        'supply_method': lambda *a: 'produce',
        'uom_id': _get_uom_id,
        'uom_po_id': _get_uom_id,
        'categ_id' : _default_category,
        'type' : lambda *a: 'consu',
    }
    _columns = {'key': fields.char('Key', size=3),
                'categ_id': fields.many2one('product.category', 'Category'),
                'sale_ok': fields.boolean('Can be Sold', help="Determines if the product can be visible in the list of product within a selection from a sale order line."),
                'purchase_ok': fields.boolean('Can be Purchased', help="Determine if the product is visible in the list of products within a selection from a purchase order line."),
                'type': fields.selection([('product','Stockable Product'),('consu', 'Consumable'),('service','Service')], 'Product Type', required=True, help="Will change the way procurements are processed. Consumables are stockable products with infinite stock, or for use when you have no inventory management in the system."),
                'supply_method': fields.selection([('produce','Produce'),('buy','Buy')], 'Supply method', required=True, help="Produce will generate production order or tasks, according to the product type. Purchase will trigger purchase orders when requested."),
                'procure_method': fields.selection([('make_to_stock','Make to Stock'),('make_to_order','Make to Order')], 'Procurement Method', required=True, help="'Make to Stock': When needed, take from the stock or wait until re-supplying. 'Make to Order': When needed, purchase or produce for the procurement request."),
                'cost_method': fields.selection([('standard','Standard Price'), ('average','Average Price')], 'Costing Method', required=True,
                                                help="Standard Price: the cost price is fixed and recomputed periodically (usually at the end of the year), Average Price: the cost price is recomputed at each reception of products."),
                'list_price': fields.float('Sale Price', digits_compute=dp.get_precision('Sale Price'), help="Base price for computing the customer price. Sometimes called the catalog price."),
                'price_margin': fields.float('Variant Price Margin', digits_compute=dp.get_precision('Sale Price')),
                'state': fields.selection([('',''),
                                           ('draft', 'In Development'),
                                           ('sellable','Normal'),
                                           ('end','End of Lifecycle'),
                                           ('obsolete','Obsolete')], 'Status', help="Tells the user if he can use the product or not."),
                'uom_id': fields.many2one('product.uom', 'Default Unit Of Measure', required=True, help="Default Unit of Measure used for all stock operation."),
                'uom_po_id': fields.many2one('product.uom', 'Purchase Unit of Measure', required=True, help="Default Unit of Measure used for purchase orders. It must be in the same category than the default unit of measure.")
                }
category_relation()

class bom_file(osv.osv):
    _inherit = 'ir.attachment'
#    _name = 'bom.file'

    def _getModuleList(self, wslist):
        modlist = []
        for ws in wslist:
            modlist.append(ws[0][2])
        return modlist

    def _getBOMData(self, cr, uid, myid, context=None):
        import base64, os, time
        mydata = base64.standard_b64decode(self.browse(cr, uid, myid).datas)
        oldumask = os.umask(077)
        tmpfname = ('/tmp/xlsfile_%.6f' %(time.time(), )).replace('.', '_') + '.xls'
        open(tmpfname, 'w').write(mydata)
        WorkSheetList = getAllWorksheets(tmpfname)
        os.remove(tmpfname)
        return cleanWorkSheetList(WorkSheetList)

    def getOldModules(self, cr, uid, attachid):
        wslist = self._getBOMData(cr, uid, attachid)
        oldprodlist = []
        prod_obj = self.pool.get('product.product')
        for ws in wslist:
            mycols = getCols(ws, [PROJECTCOLUMN, REVCOLUMN])
            modulename = mycols[0][0]
            modlist = prod_obj.search(cr, uid, [('default_code', '=', modulename)])
            if len(modlist) == 1:
                oldprodlist.append(modlist[0])
            elif len(modlist) > 1:
                raise osv.except_osv(('Error !'), ("Multiple existing modules: %s" %(modulename, )))
        del oldprodlist[0]
        return uniquify(oldprodlist)

    def getNewBOMs(self, cr, uid, attachid):
        wslist = self._getBOMData(cr, uid, attachid)
        bom_obj = self.pool.get('mrp.bom')
        newbomlist = []
        for ws in wslist[1:]:
            mycols = getCols(ws, [DOCUMENTCOLUMN, PROJECTCOLUMN, REVCOLUMN])
            if str(mycols[0][0])[:3] in DOCREVPREFIXBOM:
                bomname = '-'.join([mycols[0][1], process_zero(mycols[0][2])])
            else:
                bomname = mycols[0][1]
            bomlist = bom_obj.search(cr, uid, [('name', '=', bomname), ('bom_id', '=', False)])
            if len(bomlist) == 0:
                newbomlist.append(bomname)
            elif len(bomlist) > 1:
                raise osv.except_osv(('Error !'), ("Multiple existing BOM's: %s" %(bomname, )))
        return '\n'.join(uniquify(newbomlist))

    def getNewModules(self, cr, uid, attachid):
        wslist = self._getBOMData(cr, uid, attachid)
        newprodlist = []
        prod_obj = self.pool.get('product.product')
        for ws in wslist:
            mycols = getCols(ws, [PROJECTCOLUMN, REVCOLUMN])
            modulename = mycols[0][0]
            modlist = prod_obj.search(cr, uid, [('default_code', '=', modulename)])
            if len(modlist) == 0:
                newprodlist.append(prodname)
            elif len(modlist) > 1:
                raise osv.except_osv(('Error !'), ("Multiple existing modules: %s" %(modulename, )))
        return uniquify(newprodlist)

    def _getDataFromRow(self, cr, uid, row):
        mycode = getDefaultCodeFromRow(row)
        mydescr = row[4]
        mydict = dict(MODULODICT)
        mydict['default_code'] = mycode
        mydict['name'] = mydescr
        #mydict['description'] = mydescr
        if row[6] and row[6] != "NA":
            mydict['variants'] = row[6]
        if row[1]:
            mydict['equitek_docnum'] = row[1]
            catrel_obj = self.pool.get('category.relation')
            rowcat = row[1][:PREFIXLEN]
            idlist = catrel_obj.search(cr, uid, [('key', '=', rowcat)])
            if idlist:
                myres = catrel_obj.read(cr, uid, idlist[0], ["key",
                                                             "categ_id",
                                                             "sale_ok",
                                                             "purchase_ok",
                                                             "type",
                                                             "supply_method",
                                                             "procure_method",
                                                             "cost_method",
                                                             "list_price",
                                                             "price_margin",
                                                             "state",
                                                             "uom_id",
                                                             "uom_po_id"])
                for key in COMPLEXKEYLIST:
                    if key in myres.keys():
                        myres[key] = myres[key][0]
                mydict.update(myres)
        return mydict

    def _addModuleFromRow(self, cr, uid, row):
        mydict = self._getDataFromRow(cr, uid, row)
        prod_obj = self.pool.get('product.product')
        return prod_obj.create(cr, uid, mydict)

    def _addModuleBOMFromWS(self, cr, uid, bomid, ws):
        pmlist = []
        prod_obj = self.pool.get('product.product')
        bom_obj = self.pool.get('mrp.bom')
        for row in ws[2:]:
            default_code = getDefaultCodeFromRow(row)
            myproduct_idlist = prod_obj.search(cr, uid, [('default_code', '=', default_code)])
            if not len(myproduct_idlist):
                myproduct_id = self._addProductFromRow(cr, uid, row)
            else:
                myproduct_id = myproduct_idlist[0]
            if ('find' in dir(row[1])) and (row[1].find(PMPREFIX) == 0):
                pmlist.append(myproduct_id)
            mydescr = row[4]
            bomdict = dict(BOMDICT)
            bomdict['product_id'] = myproduct_id
            bomdict['bom_id'] = bomid
            myuomid = prod_obj.read(cr, uid, myproduct_id, ['uom_id'])['uom_id'][0]
            bomdict['product_uom'] = myuomid
            bomdict['product_qty'] = row[5]
            bomdict['name'] = default_code
            mybomid = bom_obj.create(cr, uid, bomdict)
            if len(row) >= 8 and row[7]:
                mypmid = bom_obj.search(cr, uid, [('name', '=', default_code), ('bom_id', '=', False)])
                if mypmid:
                    continue
                prod_obj.write(cr, uid, myproduct_id, {'name': row[4], 'default_code': default_code})
                bomdict = dict(BOMDICT)
                res = prod_obj.search(cr, uid, [('default_code', '=', row[6])])
                if not res:
                    raise osv.except_osv(('Error !'), ('El producto %s no existe.' %(row[6], )))
                else:
                    mysubbom_prod_id = res[0]
                bomdict['product_id'] = myproduct_id
                bomdict['bom_id'] = False
                myuomid = prod_obj.read(cr, uid, myproduct_id, ['uom_id'])['uom_id'][0]
                bomdict['product_uom'] = myuomid
                bomdict['name'] = default_code
                mysubbomid = bom_obj.create(cr, uid, bomdict)
                mysubbom_uom_id = prod_obj.read(cr, uid, mysubbom_prod_id, ['uom_id'])['uom_id'][0]
                bomdict['product_id'] = mysubbom_prod_id
                bomdict['product_qty'] = row[7]
                bomdict['bom_id'] = mysubbomid
                bomdict['product_uom'] = mysubbom_uom_id
                mysubbomid_ref = bom_obj.create(cr, uid, bomdict)
        return pmlist

        
    def loadModuleBOM(self, cr, uid, wslist, form):  ### testModule
        bom_obj = self.pool.get('mrp.bom')
        myws = wslist[0]
        mydoc = myws[0][1]
        mycode = myws[0][2]
        bomname = form['bom_name']
        bom_idlist = bom_obj.search(cr, uid, [('name', '=', bomname)])
        if len(bom_idlist):
            raise osv.except_osv(('Error !'), ('Existe ya una lista de materiales llamada %s.' %(bomname, )))
        prod_obj = self.pool.get('product.product')
        prod_idlist = prod_obj.search(cr, uid, [('default_code', '=', mycode)])
        if len(prod_idlist) == 1:
            product_id = prod_idlist[0]
        else:
            raise osv.except_osv(('Error !'), ('Existen %i definiciones del producto %s.' %(len(prod_idlist), mycode)))
        uomid = prod_obj.read(cr, uid, product_id, ['uom_id'])['uom_id'][0]
        bomdict = dict(BOMDICT)
        bomdict['product_id'] = product_id
        bomdict['product_uom'] = uomid
        bomdict['bom_id'] = False
        bomdict['name'] = bomname
        bomid = bom_obj.create(cr, uid, bomdict)
        result = {}
        result['module_bom_id'] = bomid
        result['bom_pmlist'] = self._addModuleBOMFromWS(cr, uid, bomid, myws)
        return result
        

    def _addProductFromRow(self, cr, uid, row):
        mydict = self._getDataFromRow(cr, uid, row)
        prod_obj = self.pool.get('product.product')
        myprodid = prod_obj.search(cr, uid, [('default_code', '=', mydict['default_code'])])
        return prod_obj.create(cr, uid, mydict)

    def altaMaquina(self, cr, uid, wslist, attach_id, form):
        result = {}
        myws = wslist[0]
        mycode = myws[0][2]
        prod_obj = self.pool.get('product.product')
        res = self.read(cr, uid, attach_id, ['res_model', 'res_id'])
        maqid = res['res_id']
        #bomname = prod_obj.read(cr, uid, maqid, ['name'])['name']
        bomname = form['bom_name']
        result['maquina_id'] = maqid
        modlist = self._getModuleList(wslist[1:])
        myres = self.read(cr, uid, attach_id, ['res_model', 'res_id'])
        uomid = self.pool.get(myres['res_model']).read(cr, uid, myres['res_id'], ['uom_id'])['uom_id'][0]
        bom_obj = self.pool.get('mrp.bom')
        bomdict = dict(BOMDICT)
        bomdict['product_id'] = maqid
        bomdict['name'] = bomname
        bomdict['product_uom'] = uomid
        bomdict['bom_id'] = False
        bomid = bom_obj.create(cr, uid, bomdict)
        result['bom_id'] = bomid
        for row in myws[2:]:
            mydoc = row[1]
            mycode = row[2]
            if mydoc[:3] in DOCREVPREFIXBOM:
                mybomname = '-'.join([mycode, process_zero(row[3])])
            else:
                mybomname = mycode
            mybomid = bom_obj.search(cr, uid, [('name', '=', mybomname)])
            if (mycode in modlist) and not mybomid:
                ###### Si el modulo está incluido en los worksheets anexos
                ###### Y no tiene un bom previamente dado de alta, creamos el bom
                res = prod_obj.search(cr, uid, [('default_code', '=', mycode)])
                if not len(res):
                    mymoduleid = self._addModuleFromRow(cr, uid, row)
                elif len(res) == 1:
                    mymoduleid = res[0]
                else:
                    raise osv.except_osv(('Error !'), ("Existe más de un producto con código %s" %(mycode, )))
                bomdict = dict(BOMDICT)
                bomdict['product_qty'] = row[5]
                bomdict['product_id'] = mymoduleid
                myuomid = prod_obj.read(cr, uid, mymoduleid, ['uom_id'])['uom_id'][0]
                bomdict['product_uom'] = myuomid
                bomdict['bom_id'] = bomid
                bomdict['name'] = mybomname
                mybomid = bom_obj.create(cr, uid, bomdict)
            else:
                ###### De lo contrario suponemos que el módulo ya existe (y que tiene su bom, etc).
                res = prod_obj.search(cr, uid, [('default_code', '=', mycode)])
                if len(res) == 1:
                    mymoduleid = res[0]
                elif not len(res):
                    raise osv.except_osv(('Error !'), ("El módulo %s no está dado de alta y no está incluido en el archivo a importar" %(mycode, )))
                else:
                    raise osv.except_osv(('Error !'), ("Más de un módulo %s dado de alta." %(mycode, )))
                prod_brw = prod_obj.browse(cr, uid, mymoduleid)
                bomdict = dict(BOMDICT)
                bomdict['product_id'] = mymoduleid
                myuomid = prod_obj.read(cr, uid, mymoduleid, ['uom_id'])['uom_id'][0]
                bomdict['product_uom'] =  myuomid
                bomdict['bom_id'] = bomid
                bomdict['name'] = mybomname
                mybomid = bom_obj.create(cr, uid, bomdict)
        return result

    def _addModuleFromWS(self, cr, uid, ws):
        result = {}
        result['bom_pmlist'] = []
        mydoc = ws[0][1]
        mycode = ws[0][2]
        myrev  = ws[0][3]
        myrevstr = process_zero(myrev)
        prod_obj = self.pool.get('product.product')
        product_id = prod_obj.search(cr, uid, [('default_code', '=', mycode)])[0]
        result['module_product_id'] = product_id
        if mydoc[:3] in DOCREVPREFIXBOM:
            mybomname = '-'.join([str(mycode), myrevstr])
        else:
            mybomname = str(mycode)
        bom_obj = self.pool.get('mrp.bom')
        bomname = prod_obj.read(cr, uid, product_id, ['name_template'])['name_template']
        uomid = self.pool.get('product.product').read(cr, uid, product_id, ['uom_id'])['uom_id'][0]
        bomdict = dict(BOMDICT)
        bomdict['product_id'] = product_id
        bomdict['product_uom'] = uomid
        bomdict['bom_id'] = False
        bomdict['name'] = mybomname
        bomid = bom_obj.create(cr, uid, bomdict)
        result['module_bom_id'] = bomid
        result['bom_pmlist'] = self._addModuleBOMFromWS(cr, uid, bomid, ws)
        return result


    def importMaquina(self, cr, uid, form):
        result = []
        attachid = form['xls_attach_id']
        wslist = self._getBOMData(cr, uid, attachid)
        ###  Si solamente hay un worksheet en el xml, entonces estamos dando de alta un bom para un módulo
        if len(wslist) == 1:
            rr = {'bomdata': [self.loadModuleBOM(cr, uid, wslist, form)]}
            return rr
        maqdict = self.altaMaquina(cr, uid, wslist, attachid, form)
        main_bomid = maqdict['bom_id']
        bomdata = []
        for ws in wslist[1:]:
            mydoc = ws[0][1]
            mycode = ws[0][2]
            myrev  = ws[0][3]
            myrevstr = process_zero(myrev)
            if mydoc[:3] in DOCREVPREFIXBOM:
                mybomname = '-'.join([str(mycode), myrevstr])
            else:
                mybomname = str(mycode)
            bom_obj = self.pool.get('mrp.bom')
            mybomid = bom_obj.search(cr, uid, [('name', '=', mybomname), ('bom_id', '=', False)])
            if  not mybomid:
                bomdata.append(self._addModuleFromWS(cr, uid, ws))
        rr = {'main_bomid': main_bomid,
              'bomdata': bomdata}
        return rr

bom_file()

class ir_attachment(osv.osv):
    _inherit = 'ir.attachment'

    def _name_get_resname(self, cr, uid, ids, object,method, context):
        data = {}
        for attachment in self.browse(cr, uid, ids, context=context):
            model_object = attachment.res_model
            res_id = attachment.res_id
            if model_object and res_id:
                model_pool = self.pool.get(model_object)
                res = model_pool.name_get(cr,uid,[res_id],context)
                data[attachment.id] = (res and res[0][1]) or False
            else:
                 data[attachment.id] = False
        return data

    _columns = {
        'res_name': fields.function(_name_get_resname, type='char', size=256,
                                    string='Resource Name', method=True, store=True),
        }
ir_attachment()

class mrp_bom(osv.osv):
    _inherit = 'mrp.bom'

    _columns = {
        'bom_notes': fields.text('Notes'),
        'bom_date': fields.datetime('Date')
        }

    _defaults ={
        'bom_date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S')
        }

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        if not 'bom_date' in vals.keys():
            vals['bom_date'] = time.strftime('%Y-%m-%d %H:%M:%S')
        return super(mrp_bom, self).write(cr, uid, ids, vals, context=context)

    def copy(self, cr, uid, id, default={}, context=None, done_list=[], local=False):
        if not default:
            default = {}
        default = default.copy()
        default['bom_date'] = time.strftime('%Y-%m-%d %H:%M:%S')
        return super(mrp_bom, self).copy(cr, uid, id, default, context=context)


mrp_bom()

