# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import time

from osv import fields, osv
import netsvc
import pooler
from osv.orm import browse_record, browse_null
from tools.translate import _


class machine_attachment(osv.osv_memory):
    _name = "machine.attachment"
    _description = "Machine Attachment"

    _columns = {
        'name': fields.char('Attachment Name',size=256, required=True),
        'user_id': fields.many2one('res.users', 'Owner', select=1),
        'company_id': fields.many2one('res.company', 'Company', change_default=True),
        'create_date': fields.datetime('Date Created', readonly=True),
        'write_date': fields.datetime('Date Modified', readonly=True),
        'partner_id':fields.many2one('res.partner', 'Partner', select=1),
        'file_size': fields.integer('File Size', required=True),
    }

machine_attachment()

class import_bom(osv.osv_memory):
    _name = "import.bom"
    _description = "Import BoM"

    _columns = {
        'xls_name': fields.char('Attachment Name',size=128),
        'nest_name': fields.char('Attachment Name',size=128),
        'bom_name': fields.char('BoM Name',size=128),
        'manual_name': fields.char('Attachment Name',size=128),
        'electric_name': fields.char('Attachment Name',size=128),
        'assambly_name': fields.char('Attachment Name',size=128),
        'file.xls': fields.binary('File (.xls format)'),
        'nest_file': fields.binary('Nest Diagram File'),
        'manual_file': fields.binary('Manual File'),
        'electric_file': fields.binary('Electric Diagram File'),
        'assambly_file': fields.binary('Assembly Diagram File'),
        'sheets_book_file': fields.binary('Sheets Book File'),
        'sheets_book_name': fields.char('Sheets Book Name',size=128),
        'machined_book_file': fields.binary('Machiend Book File'),
        'machined_book_name': fields.char('Machiend Book Name',size=128),
        'layout_file': fields.binary('Layout File'),
        'layout_name': fields.char('Layout Book Name',size=128),
        'neumatic_file': fields.binary('Neumatic Diagram File'),
        'neumatic_name': fields.char('Neumatic Name',size=128),
        'state': fields.selection( ( ('wizard0','wizard0'),   # Search file
                                     ('wizard1','wizard1'),   # Display Existing Modules
                                     ('wizard2','wizard2'),   # Display Existing Products
                                     ('wizard3','wizard3'),   # Asign files to Machine
                                     ('wizard4','wizard4'),   # Asign files to Machined Parts
                                     ('wizard5','wizard5'),   # Asign files to Machined Parts
                                       ) ),
        'text1': fields.text('Text'),
        'text2': fields.text('Text'),
        'line_ids': fields.one2many('machined.parts', 'import_id', 'Machinated Parts'),
        'xls_attach_id': fields.many2one('ir.attachment', 'XLS Attachment'),
        'product_id': fields.many2one('product.product', 'product'),
        #'old_module_ids': fields.many2many('product.product', 'module_old_rel', 'product_id', 'old_module_id', 'Old Modules'),
        'newbomlist': fields.text("New BOM's"),
        'attachment_ids': fields.many2many('ir.attachment', 'product_attachment_rel', 'product_id', 'attachment_id', 'Attachments'),
        'information': fields.char('Information',size=64),
    }
    _defaults = {
        'state': 'wizard0',
    }

    def _get_data(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        data = {}
        data['ids'] = context.get('active_ids', [])
        data['id'] = context and context.get('active_id', False)
        data['model'] = context.get('active_model', 'ir.ui.menu')
        data['form'] = self.read(cr, uid, ids)[0]
        return data

    def default_get(self, cr, uid, fields, context=None):
        attach_obj = self.pool.get('ir.attachment')
        if context is None:
            context = {}
        record_id = context and context.get('active_id', False) or False
        res = super(import_bom, self).default_get(cr, uid, fields, context=context)
        if record_id:
            attach_ids = attach_obj.search(cr, uid, [('res_id','=',record_id),('res_model','=', context['active_model'])])
            res.update({'product_id':record_id, 'attachment_ids': attach_ids}) 
        return res

    def onchange_product_id(self, cr, uid, ids, product_id, context=None):
        attach_obj = self.pool.get('ir.attachment')
        if product_id:
            attach_ids = attach_obj.search(cr, uid, [('res_id','=',product_id),('res_model','=', 'product.product')])
            return {'value': {'attachment_ids': attach_ids}}
        return {'value': {'attachment_ids': []}}

    def check_existing_modules(self, cr, uid, ids, context=None):
        #### Esta función ya no se utiliza. Ya no se generan nuevos BOMs para productos que ya existen y
        #### cuyo nombre aparece en las plantillas secundarias del archivo xls.
        self.BOM_DATA = []
        #self.COUNT = 1
        self.COUNT = 0
        data = self._get_data(cr, uid, ids, context=context)
        form = data['form']
        attach_id = self.attach_file(cr, uid, form['product_id'], form['file.xls'], form['xls_name'], form['bom_name'])
        #res = self.pool.get('ir.attachment').getOldModules(cr, uid, attach_id)
        res = self.pool.get('ir.attachment').getModulesToCreateBOM(cr, uid, attach_id)
        return self.write(cr, uid, ids, {'state':'wizard1', 'xls_attach_id':attach_id, 'old_module_ids':[(6,False,res)]}, context=context)

    def show_new_modules(self, cr, uid, ids, context=None):
        self.BOM_DATA = []
        #self.COUNT = 1
        self.COUNT = 0
        data = self._get_data(cr, uid, ids, context=context)
        form = data['form']
        attach_id = self.attach_file(cr, uid, form['product_id'], form['file.xls'], form['xls_name'], form['bom_name'])
        #res = self.pool.get('ir.attachment').getOldModules(cr, uid, attach_id)
        res = self.pool.get('ir.attachment').getNewBOMs(cr, uid, attach_id)
        return self.write(cr, uid, ids, {'state':'wizard1', 'xls_attach_id':attach_id, 'newbomlist':res}, context=context)

    def import_bom(self, cr, uid, ids, context=None):
#         self.BOM_DATA = []
#         self.COUNT = 0
#         data = self._get_data(cr, uid, ids, context=context)
#         form = data['form']
#         attach_id = self.attach_file(cr, uid, form['product_id'], form['file.xls'], form['xls_name'], form['bom_name'])
        mp_obj = self.pool.get('machined.parts')
        data = self._get_data(cr, uid, ids, context=context)
        #data['form']['xls_attach_id'] = attach_id
        res = self.pool.get('ir.attachment').importMaquina(cr, uid, data['form'])#['xls_attach_id'], data['form'])
        self.BOM_DATA = res['bomdata']
        if len(self.BOM_DATA):
            bom_pmlist = self.BOM_DATA[self.COUNT-1]['bom_pmlist']
            res = []
            for mp in bom_pmlist:
                res.append(mp_obj.create(cr, uid, {'import_id':data['id'], 'product_id':mp}, context=context))
            inf = str(self.COUNT + 1) + '/' + str(len(self.BOM_DATA))
            self.COUNT += 1
            return self.write(cr, uid, ids, {'state':'wizard3', 'information': inf,'line_ids':map(lambda x: (4, x, False), res)}, context=context)
        else:
            return self.write(cr, uid, ids, {'state': 'wizard5'})

    def attach_machine_files(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state':'wizard4'}, context=context)

    def attach_files(self, cr, uid, ids, context=None):
        mp_obj = self.pool.get('machined.parts')
        data = self._get_data(cr, uid, ids, context=context)
        form = data['form']
        if self.COUNT == len(self.BOM_DATA):
            if form['nest_file']: self.attach_file(cr, uid, form['product_id'], form['nest_file'], form['nest_name'])
            if form['manual_file']: self.attach_file(cr, uid, form['product_id'], form['manual_file'], form['manual_name'])
            if form['electric_file']: self.attach_file(cr, uid, form['product_id'], form['electric_file'], form['electric_name'])
            if form['assambly_file']: self.attach_file(cr, uid, form['product_id'], form['assambly_file'], form['assambly_name'])
            if form['sheets_book_file']: self.attach_file(cr, uid, form['product_id'], form['sheets_book_file'], form['sheets_book_name'])
            if form['machined_book_file']: self.attach_file(cr, uid, form['product_id'], form['machined_book_file'], form['machined_book_name'])
            if form['layout_file']: self.attach_file(cr, uid, form['product_id'], form['layout_file'], form['layout_name'])
            if form['neumatic_file']: self.attach_file(cr, uid, form['product_id'], form['neumatic_file'], form['neumatic_name'])
            return self.write(cr, uid, ids, {'state':'wizard5'}, context=context)
        bom_pmlist = self.BOM_DATA[self.COUNT-1]['bom_pmlist']
        res = []
        mp_obj.write(cr, uid, mp_obj.search(cr, uid, []), {'import_id': False})
        for mp in bom_pmlist:
            res.append(mp_obj.create(cr, uid, {'import_id':data['id'], 'product_id':mp}, context=context))
        inf = str(self.COUNT) + '/' + str(len(self.BOM_DATA)-1)
        self.COUNT += 1
        return self.write(cr, uid, ids, {'state':'wizard4', 'information': inf, 'line_ids':map(lambda x: (4, x, False), res)}, context=context)

    def save_files(self, cr, uid, ids, context=None):
        mp_obj = self.pool.get('machined.parts')
        for mp in mp_obj.browse(cr, uid, mp_obj.search(cr, uid, [])):
            if mp.file:
                self.attach_file(cr, uid, mp.product_id.id, mp.file, mp.name)
        return {}

    def attach_file(self, cr, uid, product_id, _file, name, file_name='', context=None):
        product_name = product_id and self.pool.get('product.product').browse(cr, uid, product_id).name[:126] or False
        if not file_name:
            file_name = name
        return self.pool.get('ir.attachment').create(cr, uid, {
            'name': file_name,
            'datas_fname': name,
            'res_name': product_name,
            'datas': _file,
            'res_model': 'product.product',
            'type': 'binary',
            'res_id': product_id,})


import_bom()

class machined_parts(osv.osv_memory):
    _name = "machined.parts"
    _description = "Machined Parts"

    _columns = {
        'import_id': fields.many2one('import.bom', 'Import Bom'),
        'product_id': fields.many2one('product.product', 'Product'),
        'file': fields.binary('Diagram File'),
        'name': fields.char('Attachment Name',size=128),
    }

machined_parts()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
